/*
 * Copyright (C) 2007-2015 Siemens AG
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/*******************************************************************
 *
 * @author Daniel.Peintner.EXT@siemens.com
 * @version 0.9.3 
 * @contact Joerg.Heuer@siemens.com
 *
 * <p>Code generated by EXIdizer</p>
 * <p>Schema: V2G_CI_MsgDef.xsd</p>
 *
 *
 ********************************************************************/



#include "EncoderChannel.h"

#include "EXIOptions.h"
#include "BitOutputStream.h"
#include "EXITypes.h"
#include "MethodsBag.h"
#include "ErrorCodes.h"

#ifndef BYTE_ENCODER_CHANNEL_C
#define BYTE_ENCODER_CHANNEL_C

#if EXI_OPTION_ALIGNMENT == BYTE_ALIGNMENT


int encode(bitstream_t* stream, uint8_t b) {
	int errn = 0;
#if EXI_STREAM == BYTE_ARRAY
	if ( (*stream->pos) < stream->size ) {
		stream->data[(*stream->pos)++] = b;
	} else {
		errn = EXI_ERROR_OUTPUT_STREAM_EOF;
	}
#endif /* EXI_STREAM == BYTE_ARRAY */
#if EXI_STREAM == FILE_STREAM
	if ( putc(b, stream->file) == EOF ) {
		errn = EXI_ERROR_OUTPUT_STREAM_EOF;
	}
#endif /* EXI_STREAM == FILE_STREAM */
	return errn;
}

/**
 * Encode a single boolean value. A false value is encoded as byte 0 and true
 * value is encode as byte 1.
 */
int encodeBoolean(bitstream_t* stream, int b) {
	uint8_t val = b ? 1 : 0;
	return encode(stream, val);
}


/**
 * Encode n-bit unsigned integer. The n least significant bits of parameter
 * b starting with the most significant, i.e. from left to right.
 */
int encodeNBitUnsignedInteger(bitstream_t* stream, uint16_t nbits, uint32_t val)  {
	int errn = 0;
	if (nbits > 0) {
		if (nbits < 9) {
			/* 1 byte */
			errn = encode(stream, val & 0xff);
		} else if (nbits < 17) {
			/* 2 bytes */
			errn = encode(stream, val & 0x00ff);
			if(errn == 0) {
				errn = encode(stream, (uint8_t)((val & 0xff00) >> 8));
			}
		} else if (nbits < 25) {
			/* 3 bytes */
			errn = encode(stream, val & 0x0000ff);
			if(errn == 0) {
				errn = encode(stream, (uint8_t)((val & 0x00ff00) >> 8));
				if(errn == 0) {
					errn = encode(stream, (uint8_t)((val & 0xff0000) >> 16));
				}
			}
		} else if (nbits < 33) {
			/* 4 bytes */
			errn = encode(stream, val & 0x000000ff);
			if(errn == 0) {
				errn = encode(stream, (uint8_t)((val & 0x0000ff00) >> 8));
				if(errn == 0) {
					errn = encode(stream, (uint8_t)((val & 0x00ff0000) >> 16));
					if(errn == 0) {
						errn = encode(stream, (uint8_t)((val & 0xff000000) >> 24));
					}
				}
			}
		} else {
			/* TODO Currently not more than 4 Bytes allowed for NBitUnsignedInteger */
			errn = EXI_UNSUPPORTED_NBIT_INTEGER_LENGTH;
		}
	}
	return errn;
}


/**
 * Flush underlying bit output stream.
 */
#pragma warning( disable : 4100 ) /* warning unreferenced parameter 'stream' */
int encodeFinish(bitstream_t* stream) {
	/* no pending bits in byte-aligned mode */
	return 0;
}
#pragma warning( default : 4100 ) /* warning unreferenced parameter 'stream' */

#endif /* alignment */

#endif












